// Hello world!
if not MUtils then rawset(_G, "MUtils", {}) end
MUtils.name = "MapUtils"
MUtils.fullname = "Map Utilities"
MUtils.ver = "NoClipboard (1.21)"
MUtils.author = "TheOneGoofAli"
MUtils.specialmapinfo = {}

// Unless a custom build extends our map limit... this shouldn't be touched.
local MUMapMax = CV_RegisterVar({
	name = "mumapmax",
	defaultvalue = 1035,
	flags = CV_NETVAR | CV_HIDDEN | CV_NOSHOWHELP,
	PossibleValue = {MIN = 1035, MAX = 65536}
})

// Plugin run toggle.
local MUPluginRun = CV_RegisterVar({
	name = "maputilspluginrun",
	defaultvalue = 1,
	flags = CV_SHOWMODIF,
	PossibleValue = CV_OnOff
})

// The meat and potatoes of our map list/search adventure! Now with modularity!
local function MUMapAccess(searchname, maptype)
	searchname, maptype = tostring($) or "", tostring($) or "race" // Default if not defined.
	if tonumber(searchname) != nil then searchname = "" end // Do we have a non-extended map number instead?
	if maptype != "race" and maptype != "battle" and maptype != "all" and maptype != "singleplayer" then maptype = "all" end // If gametype not selected, default to all.
	// Initialize map counts, boolean for if we are searching and the table that will be returned.
	local tmp, hmp, dosearch = 0, 0, true
	local mapoutput = {}
	if searchname == "" then dosearch = false end
	for i = 0, MUMapMax.value do
		if mapheaderinfo[i] != nil then
			if (maptype == "race" and mapheaderinfo[i].typeoflevel == TOL_RACE) // Race.
			or (maptype == "battle" and mapheaderinfo[i].typeoflevel == TOL_MATCH) // Battle.
			or (maptype == "singleplayer" and mapheaderinfo[i].typeoflevel == TOL_SP) // ...did you hide it THIS much?
			or (maptype == "all") then
				local entrydata = {}
				if (dosearch
				and (searchname.find(mapheaderinfo[i].lvlttl:lower(), searchname, 1, true) != nil) // Searching in title...
				or (searchname.find(mapheaderinfo[i].subttl:lower(), searchname, 1, true) != nil) // Searching in subtitle...
				or (searchname.find(mapheaderinfo[i].zonttl:lower(), searchname, 1, true) != nil) // Do we have a ZONE?
				or (searchname.find(mapheaderinfo[i].actnum:lower(), searchname, 1, true) != nil)) // ...the numbers...
				or not dosearch then // Or we are just listing all maps.
					local ishell, lvltype = false, ""
					if mapheaderinfo[i].menuflags & 1 == 1 then
						hmp = $+1
						ishell = true
					end
					if (mapheaderinfo[i].typeoflevel & TOL_RACE) == 8 then lvltype = "Race"
					elseif (mapheaderinfo[i].typeoflevel & TOL_MATCH) == 16 then lvltype = "Battle"
					elseif (mapheaderinfo[i].typeoflevel & TOL_SP) == 1 then lvltype = "Singleplayer"
					else lvltype = "Unknown/Other" end
					// Process plugins.
					local extramapinfo = MUtils.specialmapinfo
					local extramapinfotext
					if type(extramapinfo) == "table" and #extramapinfo > 0 and MUPluginRun.value == 1 then
						for v = 1, #extramapinfo do
							if extramapinfotext == nil then extramapinfotext = "" end
							local addtext = MUtils.specialmapinfo[v](i)
							if addtext != nil then
								if extramapinfotext == "" then
									extramapinfotext = addtext
								else
									extramapinfotext = $.." "..addtext
								end
							end
						end
					end
					// Collect everything into a local (sub)table before we add that to the main table.
					// i[1] is the level title, subtitle, zone title, act number...
					// i[2] is the direct/extended map ID (i.e. "mapXX")
					// i[3] is a boolean. True if map is considered a hellmap, false otherwise.
					// i[4] is the map number (non-extended, i.e. 5 or 789)
					// i[5] is the Type Of Level of a map.
					// i[6] is a string of additional map info should it exist.
					// If MUDebug is enabled, listing/searching for maps will also write out
					// i[1] through i[4] for a given map, be it to console or file.
					local fullmapname = mapheaderinfo[i].lvlttl + " " + mapheaderinfo[i].subttl + " " + mapheaderinfo[i].zonttl + " " + mapheaderinfo[i].actnum
					entrydata = {fullmapname:gsub("[ \t]+%f[\r\n%z]", ""), G_BuildMapName(i), ishell, i, lvltype, extramapinfotext}
					table.insert(mapoutput, entrydata)
					tmp = $+1
				end
			end
		end
	end
	return mapoutput, tmp, hmp // Returning the resulting table, total & hell-only map counts.
end
// Expose function.
MUtils.maplist = MUMapAccess

// Debug mode. Toggle for writing more than needed things.
local MUDebug = CV_RegisterVar({
	name = "mudebug",
	defaultvalue = 0,
	flags = CV_SHOWMODIFONETIME,
	PossibleValue = CV_OnOff
})

// Return a (better) string to write to console.
local function WriteMapInfoV2(p, mapinfotable)
	local zoneinfo, mapid, ishell, entry, lvltype = mapinfotable[1], mapinfotable[2], mapinfotable[3], mapinfotable[4], mapinfotable[5]
	local extrastuff
	if mapinfotable[6] != nil and mapinfotable[6] != "" then extrastuff = " "..mapinfotable[6] end
	local texty
	if ishell then
		texty = "\x82"..zoneinfo.." / "..mapid.."\x85".." - Hellmap".."\x83".." ("..entry..")".."\x88".." ("..lvltype..")"
		if extrastuff != nil then texty = $.."\x84"..extrastuff end
	else
		texty = "\x82"..zoneinfo.." / "..mapid.."\x83".." ("..entry..")".."\x88".." ("..lvltype..")"
		if extrastuff != nil then texty = $.."\x84"..extrastuff end
	end
	return texty
end
MUtils.mapinfostringen = WriteMapInfoV2

// Conversion between extended and integer map ID.
// Integer to Extended was easier...
local function INTtoEXT(i)
	local dummy = "dumdumdum"
	if i < 100 then return i end // Return itself if it doesn't have an extended map equivalent.
	if i > MUMapMax.value then return end // ...why?
	local x, p, q, a, b
	x = i-100
	p = x/36
	q = (x-(36*p))
	a = dummy.char(p+65)
	if q < 10 then b = q
	else b = dummy.char(q+55) end
	return a..b
end
MUtils.inttoext = INTtoEXT

// ...as this was an adventure of itself to figure out.
local function EXTtoINT(i)
	local dummy = "mudmudmud"
	if i.len(i) != 2 then return end // Do we have an extended map ID without the MAP bit?
	if tonumber(i) != nil and tonumber(i) < 100 then return i end // If the ID is a number... why? Return itself.
	i = tostring(i:lower())
	local x, y, p, q
	local ttable = {}
	for k, v in dummy.gmatch(i, "%w") do
		table.insert(ttable, k)
	end
	x, y = ttable[1], ttable[2]
	if tonumber(x) != nil then return end // Is our first character a number?!
	p = dummy.byte(x)-97
	if tonumber(y) != nil then q = y
	else q = dummy.byte(y)-87 end
	return ((36*p+q)+100)
end
MUtils.exttoint = EXTtoINT

// Show maps or maps that match the search term.
local function ConsoleListMap(p, zonename, gmode)
	zonename = tostring($) or ""
	local maptable, totalmapcount, hellmapcount = MUtils.maplist(zonename, gmode)
	if #maptable < 1 then
		if zonename == "" then
			CONS_Printf(p, "\x82"..MUtils.name..":".."\x85".." The returned table doesn't have any maps. Is the function modified?".."\x80")
		else
			CONS_Printf(p, "\x82"..MUtils.name..":".."\x85".." Nothing was found.".."\x80")
		end
		return
	else
		CONS_Printf(p, "\x82"..MUtils.name..":".."\x80".." Found:")
		for k, i in ipairs(maptable) do
			CONS_Printf(p, MUtils.mapinfostringen(p, i))
			if MUDebug.value then CONS_Printf(p, tostring(i[1]).." ", tostring(i[2]).." ", tostring(i[3]).." ", tostring(i[4]).." ", tostring(i[5]), tostring(i[6])) end
		end
	end
	CONS_Printf(p, "\x82"..MUtils.name..":".."\x80".." Map count: ".."\x81"..totalmapcount-hellmapcount.."\x80"..", of which ".."\x85"..hellmapcount.."\x80".." are hellmaps.",
				   "\x82"..MUtils.name..":".."\x80".." Total map count: ".."\x81"..totalmapcount.."\x80")
end

// Write plugins active to open file handle.
local function WritePluginsActive(table, file)
	if #table > 0 then
		for i = 1, #table
			file:write(table[i](nil, true), "\n")
		end
	end
end

// Dump map list/search result to file.
local function MUOutputFile(p, filename, zname, gmode)
	if p != consoleplayer then return end // No io.openlocal... we'll have to take this approach.
	local maptable, totalmapcount, hellmapcount = MUtils.maplist(zname, gmode)
	if maptable != nil and #maptable < 1 then
		if zname == "" then
			CONS_Printf(p, "\x82"..MUtils.name..":".."\x85".." No maps found.".."\x80")
		else
			CONS_Printf(p, "\x82"..MUtils.name..":".."\x85".." Nothing was found, aborting write.".."\x80")
		return
		end
	else
		local external = io.open(filename, "w+")
		CONS_Printf(p, "\x82"..MUtils.name..":".."\x80".." Writing to file luafiles/"..filename.."...")
		local plugintable = MUtils.specialmapinfo
		external:write("Generated with MapUtils "..MUtils.ver.." by TheOneGoofAli.", "\n")
		if type(plugintable) == "table" and #plugintable > 0 then
			external:write("Plugins active at time of generation:", "\n")
			WritePluginsActive(plugintable, external)
		end
		external:write("\n", "Layout:", "\n", "LevelTitle SubTitle ZoneName ActNum / MAPXX (- Hellmap if true) (Non-extended map ID) (TypeOfLevel) (Potential information from plugins)", "\n")
		for k, i in ipairs(maptable) do
			local theline = i[1].." / "..i[2]
			if i[3] then theline = $ + " - Hellmap" end
			theline = $ + " ("..i[4]..")".." ("..i[5]..")"
			if i[6] != nil and i[6] != "" then theline = $.." "..i[6] end
			if MUDebug.value then external:write(tostring(i[1]).." ", tostring(i[2]).." ", tostring(i[3]).." ", tostring(i[4]).." ", tostring(i[5]).." ", tostring(i[6]).." ", "\n") end
			external:write(theline, "\n")
		end
		external:close()
		CONS_Printf(p, "\x82"..MUtils.name..":".."\x80".." Writing to luafiles/"..filename.." complete.")
	end
end

// Where do you want to go today?
local function GoToMapCheck(p, zname, confirm, ignoregametype)
	local force = ""
	if ignoregametype == "force" or ignoregametype == "-force"
	or confirm == "-force" or confirm == "force" then // Are we forcing a map regardless of gametype?
		force = " -force"
	end
	if tonumber(zname) != nil then
		CONS_Printf(p, "Redirecting non-extended map ID to 'map'...")
		COM_BufInsertText(p, "map "..zname..force)
		return
	end
	local entry
	if tonumber(confirm) != nil then // Is the 3rd argument a number in the string?
		entry = tonumber(confirm)
	end
	local maptable, tmp, hmp = MUtils.maplist(zname)
	if #maptable < 1 then // No maps?
		if zname == "" then
			CONS_Printf(p, "\x82"..MUtils.name..":".."\x85".." No maps found... somehow?".."\x80")
		else
			CONS_Printf(p, "\x82"..MUtils.name..":".."\x85".." Nothing was found.".."\x80")
		end
		return
	elseif #maptable > 1 and zname == "" then
		CONS_Printf(p, "\x82"..MUtils.name..":".."\x80".." No search term was given.")
	else
		if #maptable == 1 then // One map found? Go!
			CONS_Printf(p, "No duplicates were found, executing 'map "..maptable[1][2]..force.."'...")
			COM_BufInsertText(p, "map "..maptable[1][2]..force)
			return
		elseif #maptable > 1 then // Multiple entries or we are executing map selection via non-extended numbering somehow?
			if tonumber(entry) != nil and tonumber(entry) <= #maptable then COM_BufInsertText(p, "map "..maptable[entry][2]..force) return
			else
				CONS_Printf(p, "\x82"..MUtils.name..":".."\x80".." Multiple maps detected with the given search term. Select the desired map by adding the last/red number to command.")
				for k, i in ipairs(maptable) do
					local text = MUtils.mapinfostringen(p, i)
					CONS_Printf(p, text.." \x85"..k)
				end
			end
		else
			CONS_Printf(p, "\x82"..MUtils.name..":".."\x80".." An non-fatal script error has occured. Aborting function.", maptable, #maptable)
		end
	end
end

// Force random hellmap.
local function GoToHellmap(p, gmode, ignoregametype)
	gmode = $ or "race"
	local force = ""
	if ignoregametype == "force" or ignoregametype == "-force" then // Are we forcing a hellmap regardless of gametype?
		force = " -force"
	end
	// Retrieve map info...
	local maptable, _, _ = MUtils.maplist("", gmode)
	if #maptable >= 1 then
		// Initialize another table just for our hellmaps.
		// v[3] is the boolean.
		local hellmaptable = {}
		for k, v in ipairs(maptable) do
			if v[3] then table.insert(hellmaptable, v) end
		end
		if #hellmaptable >= 1 then
			if #hellmaptable == 1 then print("Warping to the only hellmap detected...") else print("Warping to a random hellmap...") end
			COM_BufInsertText(p, "map "..hellmaptable[P_RandomRange(1, #hellmaptable)][2]..force)
		else
			CONS_Printf(p, "\x82"..MUtils.name..":".."\x80".." No hellmaps were found. Did you SOC them all back to normal rotation?", hellmaptable, #hellmaptable)
		end
	else
		CONS_Printf(p, "\x82"..MUtils.name..":".."\x80".." An non-fatal script error has occured. Aborting function.", maptable, #maptable)
	end
end

// Map scaling stuff. Hi, Delf! tall.lua meets MU!.. and 2.1/Kart.
// Please be careful using it as you may end up having textures that are
// Z-fighting with their rapid flickering, potentially causing epilepsy
// or seizure. Administrator/player discretion is advised.

// Toggle for vertical scale.
local MUDoScale = CV_RegisterVar({
	name = "mudozscale",
	defaultvalue = 0,
	flags = CV_NETVAR | CV_SHOWMODIF | CV_CALL,
	PossibleValue = CV_OnOff,
	func = function(cvar)
		if cvar.value == 1 then
			print("\x82"..MUtils.name..":".."\x80".." Vertical map scaling enabled.",
				  "Potential epilepsy/seizure/crash warning.")
		end
	end
})

// Scaling amount if enabled. Since the engine doesn't accept floating-point
// values directly, the CV_FLOAT flag converts our human-readable numbers into something
// the engine can handle. This is a one way process, though.
local MUScale = CV_RegisterVar({
	name = "muzscale",
	defaultvalue = 1,
	flags = CV_NETVAR | CV_SHOWMODIF | CV_FLOAT,
	PossibleValue = CV_Unsigned
})

// Actual script. Edited to work with 2.1/Kart maps.
local function MUTallLua()
	if MUDoScale.value == 0 then return end // Enabled?
	if MUScale.value == FRACUNIT then return end // Don't do 1x scaling, silly.
	print("\x85".."WARNING: ".."\x80".."The scaling script is in effect. You may experience desyncs or be unable to complete the map.")
	for sector in sectors.iterate do
		sector.floorheight = FixedMul($, MUScale.value)
		sector.ceilingheight = FixedMul($, MUScale.value)
		if sector.f_slope and (sector.f_slope.flags & SL_NODYNAMIC) then // SL_DYNAMIC isn't a thing with 2.1/Kart.
			sector.f_slope.zdelta = FixedMul($, MUScale.value)
			sector.f_slope.o = {x = sector.f_slope.o.x, y = sector.f_slope.o.y, z = FixedMul(sector.f_slope.o.z, MUScale.value)}
		end
		if sector.c_slope and (sector.c_slope.flags & SL_NODYNAMIC) then // SL_DYNAMIC isn't a thing with 2.1/Kart.
			sector.c_slope.zdelta = FixedMul($, MUScale.value)
			sector.c_slope.o = {x = sector.c_slope.o.x, y = sector.c_slope.o.y, z = FixedMul(sector.c_slope.o.z, MUScale.value)}
		end
	end
	for side in sides.iterate do
		side.rowoffset = FixedMul($, MUScale.value)
	end
	for mo in thinkers.iterate("mobj") do // Different way of iterating through mobjs.
		if mo and mo.valid then
			mo.z = FixedMul($, MUScale.value)
		end
	end
end
addHook("MapLoad", MUTallLua)

// NoClippy compatibility.
local function NoClippyCompat(p, x, y, z, angle)
	angle = $ or p.mo.angle
	if NoClippy and p.noclippy then
		p.curx = x
		p.cury = y
		p.curz = z
		p.mo.angle = angle
	else
		P_SetOrigin(p.mo, x, y, z)
		p.mo.angle = angle
	end
end
MUtils.noclippycompat = NoClippyCompat

// Do we have extensions to our script loaded?
local function MUListPlugins(p)
	local plugins = MUtils.specialmapinfo
	if type(plugins) == "table" and #plugins > 0 then
		CONS_Printf(p, "\x82"..MUtils.name..":".."\x80".." Currently loaded extensions:")
		for i = 1, #plugins do
			CONS_Printf(p, plugins[i](nil, true))
		end
	else
		CONS_Printf(p, "\x82"..MUtils.name..":".."\x80".." No extensions are loaded.")
	end
end

// Our main command function.
local function MainMU(p, arg1, arg2, arg3, arg4, arg5)
	arg1, arg2, arg3, arg4, arg5 = $ or "", $ or "", $ or "", $ or "", $ or "" // Do default values.
	arg1, arg2, arg3, arg4, arg5 = arg1:lower(), arg2:lower(), arg3:lower(), arg4:lower(), arg5:lower() // oops, all lowercase.
	if arg1 == "list" or arg1 == "l" then
		// arg2 - whether to write to console or file.
		// arg3 - desired map name (full or partial). Performs a search if defined.
		// arg4 - desired gamemode (Race or Battle).
		// Note: If you want to list maps of a specific gamemode, do:
		// maputils list <console/file> "" <race/battle/singleplayer/all>
		if arg2 == "console" or arg2 == "c" then
			// Outputting to console, here we go!
			ConsoleListMap(p, arg3, arg4)
		elseif arg2 == "file" or arg2 == "f" then
			// Guess we're writing to file...
			local filename = "maplist.txt"
			MUOutputFile(p, filename, arg3, arg4)
		else
			CONS_Printf(p, "\x82"..MUtils.name..":".."\x80".." Invalid syntax.")
		end
	elseif arg1 == "goto" or arg1 == "g" then
		// arg2 - desired map name (full or partial).
		// arg3 - entry number if multiple maps that have 'arg2' in their name.
		// arg4 - force map i.e. -force for 'map'.
		if (IsPlayerAdmin(p) or p == server) then GoToMapCheck(p, arg2, arg3, arg4)
		else CONS_Printf(p, "Only the server or a remote admin can use this.") end
	elseif arg1 == "hellmap" or arg1 == "hm" then
		// arg2 - desired gamemode. Default is "race".
		// arg3 - force hellmap even if gamemode doesn't match. "force" or "-force" are accepted.
		if (IsPlayerAdmin(p) or p == server) then GoToHellmap(p, arg2, arg3)
		else CONS_Printf(p, "Only the server or a remote admin can use this.") end
	elseif arg1 == "verify" or arg1 == "v" then
		// Did someone change the function to be something else?!
		if MUtils.maplist != MUMapAccess or MUtils.mapinfostringen != WriteMapInfoV2 then
			if arg2 == "restore" then
				if (IsPlayerAdmin(p) or p == server) then
					// Pull over, that function too fat-- err, not matching.
					MUtils.maplist = MUMapAccess
					MUtils.mapinfostringen = WriteMapInfoV2
					CONS_Printf(p, "\x82"..MUtils.name..":".."\x83".." The function(s) has/have been restored.".."\x80")
				else
					CONS_Printf(p, "Only the server or a remote admin can use this.")
				end
			else
				CONS_Printf(p, "\x82"..MUtils.name..":".."\x85".." The function(s) has/have been modified!".."\x80",
							   "If this is not intended, please contact the adminstrator of this server!")
			end
		else CONS_Printf(p, "\x82"..MUtils.name..":".."\x80".." The function(s) has/have not been modified.") end
	elseif arg1 == "inttoext" or arg1 == "toext" then // Integer to extended.
		arg2 = tonumber($)
		if arg2 != nil and arg2 >= 0 and arg2 <= MUMapMax.value then
			if arg2 < 100 then
				CONS_Printf(p, "\x82"..MUtils.name..":".."\x80".." This does not yield a corresponding extended map number.")
			else
				CONS_Printf(p, "\x82"..MUtils.name..":".."\x80".." "..tostring(arg2).." is "..MUtils.inttoext(arg2)..".")
			end
		else
			CONS_Printf(p, "\x82"..MUtils.name..":".."\x80".." Unrecognised non-extended map ID.")
		end
	elseif arg1 == "exttoint" or arg1 == "toint" then // Extended to integer.
		arg2 = tostring($)
		arg2 = arg2:upper()
		local convertresult = EXTtoINT(arg2)
		if convertresult != nil then
			CONS_Printf(p, "\x82"..MUtils.name..":".."\x80".." "..tostring(arg2).." is "..convertresult..".")
		else
			CONS_Printf(p, "\x82"..MUtils.name..":".."\x80".." Unrecognised extended map ID.")
		end
	elseif arg1 == "plugin" or arg1 == "plugins" or arg1 == "p" or arg1 == "extensions" then
		// We got more stuff attached onto us!
		MUListPlugins(p)
	elseif arg1 == "position" or arg1 == "pos" then
		// Map position stuff. Not your place in the race.
		if arg2 == "list" then
			if p.mo and p.mo.valid then
				CONS_Printf(p, "\x82"..MUtils.name..":".."\x80".." X: "..p.mo.x>>FRACBITS.." Y: "..p.mo.y>>FRACBITS.." Z: "..p.mo.z>>FRACBITS)
			else
				CONS_Printf(p, "\x82"..MUtils.name..":".."\x80".." Player object not found. Are you spectating/dead?")
			end
		elseif arg2 == "save" or arg2 == "s" then
			if p.mo and p.mo.valid then
				if (IsPlayerAdmin(p) or p == server) then
					p.musavex = p.mo.x
					p.musavey = p.mo.y
					p.musavez = p.mo.z
					p.musaveang = p.mo.angle
					CONS_Printf(p, "\x82"..MUtils.name..":".."\x80".." Current player object position saved.")
				else
					CONS_Printf(p, "Only the server or a remote admin can use this.")
				end
			else
				CONS_Printf(p, "\x82"..MUtils.name..":".."\x80".." Player object not found. Are you spectating/dead?")
			end
		elseif arg2 == "load" or arg2 == "l" then
			if p.mo and p.mo.valid then
				if (IsPlayerAdmin(p) or p == server) then
					if (p.musavex != nil and p.musavey != nil and p.musavez != nil and p.musaveang != nil) then
						MUtils.noclippycompat(p, p.musavex, p.musavey, p.musavez, p.musaveang)
						CONS_Printf(p, "\x82"..MUtils.name..":".."\x80".." Loaded saved player object position.")
					else CONS_Printf(p, "\x82"..MUtils.name..":".."\x80".." Missing saved player object position information.") end
				else
					CONS_Printf(p, "Only the server or a remote admin can use this.")
				end
			else
				CONS_Printf(p, "\x82"..MUtils.name..":".."\x80".." Player object not found. Are you spectating/dead?")
			end
		elseif arg2 == "teleport" or arg2 == "t" then
			if p.mo and p.mo.valid then
				if (IsPlayerAdmin(p) or p == server) then
					arg3, arg4, arg5 = tonumber($) or p.mo.x, tonumber($) or p.mo.y, tonumber($) or p.mo.z
					if (arg3 != nil and arg4 != nil and arg5 != nil) then
						MUtils.noclippycompat(p, arg3<<FRACBITS, arg4<<FRACBITS, arg5<<FRACBITS)
						CONS_Printf(p, "\x82"..MUtils.name..":".."\x80".." Set player object position.")
					else CONS_Printf(p, "\x82"..MUtils.name..":".."\x80".." One or more coordinates given is invalid.") end
				else
					CONS_Printf(p, "Only the server or a remote admin can use this.")
				end
			else
				CONS_Printf(p, "\x82"..MUtils.name..":".."\x80".." Player object not found. Are you spectating/dead?")
			end
		end
	else
		// List our commands!
		if arg1 == nil or arg1 == "" then
			CONS_Printf(p, "\x82"..MUtils.fullname.." "..MUtils.ver.."\x80".." by ".."\x87"..MUtils.author.."\x80",
						   "A map-related library/script with listing/searching functionality with vertical map scaling capability.")
			local cmdlist = {
				{"maputils list <console/file> [zonename] [race/battle/singleplayer/all]", " - search/list maps to console or file. To list all maps of one mode, set ".."\x83".."[zonename]".."\x80".." as \"\"."},
				{"maputils goto <zonename> [entry] [force]", " - search for and warp to a map. If there's multiple results, an ".."\x83".."[entry]".."\x80".." number can be used to select one of them."},
				{"maputils hellmap [race/battle/singleplayer/all] [force]", " - roll the dice and warp to a random hellmap."},
				{"maputils inttoext/exttoint <map ID>", " - convert integer <=> extended map numbers."},
				{"maputils verify [restore]", " - check if the main map function is altered. If so, administrators can [restore] it back to normal."},
				{"maputils plugins", " - check if any extensions are loaded."},
				{"maputils position <list/save/load/teleport> [destx] [desty] [destz]", " - list/save current, load last saved or teleport to a specific location."},
				{"mudozscale <0/1/off/on>", " - admin-only CVar. Turn map/object scaling on or off. ".."\x85".."WARNING: Potential epilepsy/seizure/server instability! Some maps may become unbeatable and/or cause desyncs!"},
				{"muzscale <scale>", " - admin-only CVar. Vertical map scale multiplier. Can take floating-point values like ".."\x83".."0.25".."\x80".." or ".."\x83".."2.1".."\x80".."."}
			}
			for i = 1, #cmdlist do
				CONS_Printf(p, "\x82"..cmdlist[i][1].."\x80"..cmdlist[i][2])
			end
		else
			CONS_Printf(p, "\x82"..MUtils.name..":".."\x80".." Invalid syntax.")
		end
	end
end

// Different ways of accessing the main command.
COM_AddCommand("maputils", MainMU)
COM_AddCommand("mu", MainMU)

// We are loaded! Print our presence!
print("\x82"..MUtils.fullname.." "..MUtils.ver.."\x80".." by ".."\x87"..MUtils.author.."\x80")

// Plugin integration from PluginExample.
// Here, we check if Daytona Kart is loaded and a map has times set for it.
// Listing it is MUDaytonaKart(nil, true).
local function MUDaytonaKartCompat(mapid, name, p)
	if name then return "Daytona Kart Compatibility" end
	if daytonakart and maptimes and mapheaderinfo[mapid].arcadetime != nil and mapheaderinfo[mapid].arcadebonus != nil then
		return "(Daytona Kart)" // Return a string for our presence.
	end
end
table.insert(MUtils.specialmapinfo, MUDaytonaKartCompat)