// Look at all those chickens-- character skins.
if not SkInfo then rawset(_G, "SkInfo", {}) end
SkInfo.name = "SkInfo"
SkInfo.fullname = "Skin Information"
SkInfo.ver = "SkInsight (1.21b)"
SkInfo.author = "TheOneGoofAli"
SkInfo.specialskinattrib = {}
SkInfo.specialinspect = {}

// Plugin run toggle.
local SkPluginRun = CV_RegisterVar({
	name = "skinfopluginrun",
	defaultvalue = 1,
	flags = CV_SHOWMODIF,
	PossibleValue = CV_OnOff
})

// Do we check speed/weight value of skin or player?
local SkInspectType = CV_RegisterVar({
	name = "skinfoinspecttype",
	defaultvalue = 1,
	flags = CV_SHOWMODIF,
	PossibleValue = {skin=0, player=1}
})

// Perform skin list. Search instead if we give it a name. A bit more simple than MapUtils's.
// This doesn't do the search by the character's stats... which is why we have a function below
// this one that does that instead.
local function SISkinAccess(searchname)
	searchname = tostring($) or ""
	local fs, ts, dosearch = 0, 0, true
	local skinlist = {}
	if searchname == "" then dosearch = false end
	for i = 0, #skins-1 do // Going beyond #skins-1 results in an invalid value.
		if skins[i] != nil and skins[i].valid then
			local entrydata = {}
			if (dosearch and (searchname.find(skins[i].name:lower(), searchname, 1, true) != nil) // Internal name search...
			or (searchname.find(skins[i].realname:lower(), searchname, 1, true) != nil)) // Real name search...
			or not dosearch then // Or we are simply listing all skins...
				// Collect everything into a local (sub)table before we add that to the main table.
				// i[1] is internal skin name.
				// i[2] is the "real" skin name as shown in Player Setup.
				// i[3] is the skin's speed value.
				// i[4] is the skin's weight value.
				// i[5] is the skin's ID. Assigned by the game as it loads characters. Can be different for
				// the same character on multiple other servers!
				// i[6] is additional information about the skin.
				local skID, extraskinattrib = skins[i], SkInfo.specialskinattrib
				local extraskinfotext
				if type(extraskinattrib) == "table" and #extraskinattrib > 0 and SkPluginRun.value == 1 then
					for v = 1, #extraskinattrib do
						if extraskinfotext == nil then extraskinfotext = "" end
						local addtext = SkInfo.specialskinattrib[v](skins[i].name)
						if addtext != nil then
							if extraskinfotext == "" then
								extraskinfotext = addtext
							else
								extraskinfotext = $.." "..addtext
							end
						end
					end
				end
				entrydata = {skID.name, skID.realname, skID.kartspeed, skID.kartweight, #skID, extraskinfotext}
				table.insert(skinlist, entrydata)
				fs = $+1
			end
			ts = $+1
		end
	end
	return skinlist, fs, ts // Returning the resulting table, amount of found skins and total skin count.
end
SkInfo.skinlist = SISkinAccess

// Obtain skins that match our request. Usually the statsearchvalue is 0 through 9, but
// if Lua was used to change it to an another value outside of that range, we can search
// for that too. Search for if statsearchvalue is equal to statsearchtype... well, type.
// "s"/"speed" searches for speed, "w"/"weight" searches for weight. "b"/"both" searches
// for skins that have speed AND weight be the same value.
local function SISkinStatFilter(p, skinlist, statsearchtype, statsearchvalue)
	if not ((statsearchtype == "speed" or statsearchtype == "s")
	or (statsearchtype == "weight" or statsearchtype == "w")
	or (statsearchtype == "both" or statsearchtype == "b")
	and tonumber(statsearchvalue) != nil) then
		CONS_Printf(p, "\x82"..SkInfo.name..":".."\x80".." Invalid syntax, returning input table...")
		return skinlist // Unexpected or no arguments given, returning table itself.
	end
	// If all is well, initialize a new table and perform search by stat instead.
	local outputlist, fs = {}, 0
	for k, i in ipairs(skinlist) do
		if ((statsearchtype == "speed" or statsearchtype == "s") and statsearchvalue == i[3]) // Speed.
		or ((statsearchtype == "weight" or statsearchtype == "w") and statsearchvalue == i[4]) // Weight.
		or ((statsearchtype == "both" or statsearchtype == "b") and (statsearchvalue == i[3] and statsearchvalue == i[4])) then // Both.
			table.insert(outputlist, i)
			fs = $+1
		end
	end
	return outputlist, fs // Return the filtered table and amount of found skins.
end
SkInfo.skinstatfilter = SISkinStatFilter

// Debug mode. Affects listing all (or searched) skins to console/file.
local SIDebug = CV_RegisterVar({
	name = "sidebug",
	defaultvalue = 0,
	flags = CV_SHOWMODIFONETIME,
	PossibleValue = CV_OnOff
})

// Return a (better) string to display.
// This was a trainwreck to finish due to adding modularity.
// RIP original WriteSkinInfo...
local function WriteSkinInfoV2(p, skindetails, plain)
	// Initialize strings to modify...
	local intname, realname, speed, weight, idskin, extrainfo
	local text
	// Unwrap the table contents of a skin... and add color if necessary.
	if plain then
		intname, realname = skindetails[1], skindetails[2]
		speed, weight = skindetails[3], skindetails[4]
		idskin = "("..skindetails[5]..")"
		//if skindetails[6] != nil then extrainfo = skindetails[6] end
		if tostring(skindetails[6]) != "nil" and skindetails[6] != "" then // Do we actually have a string to print?
			extrainfo = tostring(skindetails[6])
			text = realname.." / "..intname.." ".."Speed: "..speed.." ".."Weight: "..weight.." "..idskin.." "..extrainfo
		else // If not, just skip.
			text = realname.." / "..intname.." ".."Speed: "..speed.." ".."Weight: "..weight.." "..idskin
		end
	else
		intname, realname = "\x82"..skindetails[1], "\x82"..skindetails[2]
		speed, weight = "\x88"..skindetails[3], "\x87"..skindetails[4]
		idskin = "\x83".."("..skindetails[5]..")"
		//if skindetails[6] != nil then extrainfo = "\x84"..skindetails[6] end
		if tostring(skindetails[6]) != "nil" and skindetails[6] != "" then // Do we actually have a string to print?
			extrainfo = "\x84"..tostring(skindetails[6])
			text = realname.." / "..intname.." ".."Speed: "..speed.." ".."Weight: "..weight.." "..idskin.."\x80".." "..extrainfo
		else // If not, just skip.
			text = realname.." / "..intname.." ".."Speed: "..speed.." ".."Weight: "..weight.." "..idskin.."\x80"
		end
	end
	// Return that string!
	return text
end
SkInfo.skinfostringgen = WriteSkinInfoV2

// Write to console/log.
local function ConsoleSkinList(p, skinname, statsearchtype, statsearchvalue)
	skinname = tostring($) or ""
	local statspec, finalskintable = false, {}
	if ((statsearchtype == "speed" or statsearchtype == "s")
	or (statsearchtype == "weight" or statsearchtype == "w")
	or (statsearchtype == "both" or statsearchtype == "b") and statsearchvalue != nil) then
		if tonumber(statsearchvalue) != nil then
			statsearchvalue = tonumber($)
			statspec = true
		else CONS_Printf(p, "\x82"..SkInfo.name..":".."\x80".." Invalid syntax.") return end
	end
	local tempskintable, _, ts = SkInfo.skinlist(skinname)
	if #tempskintable < 1 then
		if skinname == "" then
			CONS_Printf(p, "\x82"..SkInfo.name..":".."\x85".." The returned table doesn't have any skins. Is the function modified?".."\x80")
		else
			CONS_Printf(p, "\x82"..SkInfo.name..":".."\x85".." Nothing was found.".."\x80")
		end
		return
	else
		if skinname != "" or statspec then
			CONS_Printf(p, "\x82"..SkInfo.name..":".."\x80".." Found:")
		end
		if statspec then finalskintable = SkInfo.skinstatfilter(p, tempskintable, statsearchtype, statsearchvalue)
		else finalskintable = tempskintable end
		for k, i in ipairs(finalskintable) do
			CONS_Printf(p, SkInfo.skinfostringgen(p, i, false))
			if SIDebug.value then CONS_Printf(p, tostring(i[1]).." ", tostring(i[2]).." ", tostring(i[3]).." ", tostring(i[4]).." "..tostring(i[5]).." "..tostring(i[6])) end
		end
	end
	CONS_Printf(p, "\x82"..SkInfo.name..":".."\x80".." Skin count: ".."\x81"..#finalskintable.."\x80",
				   "\x82"..SkInfo.name..":".."\x80".." Total skin count: ".."\x81"..ts.."\x80")
end

// Write plugins active to open file handle.
local function WritePluginsActive(table, file)
	if #table > 0 then
		for i = 1, #table
			file:write(table[i](nil, true), "\n")
		end
	end
end

// Write desired skin list to file.
local function SIOutputFile(p, filename, skinname, statsearchtype, statsearchvalue)
	if p != consoleplayer then return end // No io.openlocal... we'll have to take this approach.
	skinname = tostring($) or ""
	local statspec, finalskintable = false, {}
	if ((statsearchtype == "speed" or statsearchtype == "s")
	or (statsearchtype == "weight" or statsearchtype == "w")
	or (statsearchtype == "both" or statsearchtype == "b") and statsearchvalue != nil) then
		if tonumber(statsearchvalue) != nil then
			statsearchvalue = tonumber($)
			statspec = true
		else CONS_Printf(p, "\x82"..SkInfo.name..":".."\x80".." Invalid syntax.") return end
	end
	local tempskintable, _, ts = SkInfo.skinlist(skinname)
	if #tempskintable < 1 then
		if skinname == "" then
			CONS_Printf(p, "\x82"..SkInfo.name..":".."\x85".." The returned table doesn't have any skins, aborting write. Is the function modified?".."\x80")
		else
			CONS_Printf(p, "\x82"..SkInfo.name..":".."\x85".." Nothing was found, aborting write.".."\x80")
		end
		return
	else
		if statspec then finalskintable = SkInfo.skinstatfilter(p, tempskintable, statsearchtype, statsearchvalue)
		else finalskintable = tempskintable end
		local external = io.open(filename, "w+")
		local plugintable = SkInfo.specialskinattrib
		CONS_Printf(p, "\x82"..SkInfo.name..":".."\x80".." Writing to file luafiles/"..filename.."...")
		external:write("Generated with SkInfo "..SkInfo.ver.." by TheOneGoofAli.", "\n")
		if type(plugintable) == "table" and #plugintable > 0 then
			external:write("Plugins active at time of generation:", "\n")
			WritePluginsActive(plugintable, external)
		end
		external:write("\n", "Layout:", "\n", "<Real Skin Name> / <Internal Skin Name> Speed: <speed> Weight <weight> (SkinID) (Extra skin info - can be filled by other scripts)", "\n")
		for k, i in ipairs(finalskintable) do
			local theline = SkInfo.skinfostringgen(p, i, true)
			if SIDebug.value then external:write(tostring(i[1]).." ", tostring(i[2]).." ", tostring(i[3]).." ", tostring(i[4]).." "..tostring(i[5]).." "..tostring(i[6]), "\n") end
			external:write(theline, "\n")
		end
		external:close()
		CONS_Printf(p, "\x82"..SkInfo.name..":".."\x80".." Writing to luafiles/"..filename.." complete.")
	end
end

// ForceSkin helper.
local function SIForceSkin(p, skinname, entry)
	if skinname == "off" or tonumber(skinname) == -1 then // We're just disabling it here...
		COM_BufInsertText(p, "forceskin -1")
		return
	else
		local skID
		if tonumber(entry) != nil then skID = tonumber(entry) end
		local skintable, fs, _ = SkInfo.skinlist(skinname)
		if #skintable < 1 then
			if skinname == "" then
				CONS_Printf(p, "\x82"..SkInfo.name..":".."\x85".." The returned table doesn't have any skins. Is the function modified?".."\x80")
			else
				CONS_Printf(p, "\x82"..SkInfo.name..":".."\x85".." Nothing was found".."\x80")
			end
			return
		else
			if #skintable == 1 then
				CONS_Printf(p, "\x82"..SkInfo.name..":".."\x80".." Found one match, restricting skins to:")
				CONS_Printf(p, SkInfo.skinfostringgen(p, skintable[1], false))
				COM_BufInsertText(p, "forceskin "..skintable[1][1])
			elseif #skintable > 1 then
				if skID != nil then
					CONS_Printf(p, "\x82"..SkInfo.name..":".."\x80".." Performing skin restrction to:")
					CONS_Printf(p, SkInfo.skinfostringgen(p, skintable[skID], false))
					COM_BufInsertText(p, "forceskin "..skintable[skID][1])
				else
					CONS_Printf(p, "\x82"..SkInfo.name..":".."\x80".." Multiple skins found with the given search term. Select the desired character by adding the last/red number to command.")
					for k, i in ipairs(skintable) do
						local text = SkInfo.skinfostringgen(p, i, false)
						CONS_Printf(p, text.." \x85"..k)
					end
				end
			else
				CONS_Printf(p, "\x82"..SkInfo.name..":".."\x80".." An non-fatal script error has occured. Aborting function.", skintable, #skintable)
			end
		end
	end
end

// Skin helper. Not-so-Force.
local function SISkin(p, skinname, entry)
	if p == server and isdedicatedserver then error("Oops, can't let you do this from dedicated console!") end
	local skID
	if tonumber(entry) != nil then skID = tonumber(entry) end
	local skintable, fs, _ = SkInfo.skinlist(skinname)
	if #skintable < 1 then
		if skinname == "" then
			CONS_Printf(p, "\x82"..SkInfo.name..":".."\x85".." The returned table doesn't have any skins. Is the function modified?".."\x80")
		else
			CONS_Printf(p, "\x82"..SkInfo.name..":".."\x85".." Nothing was found".."\x80")
		end
		return
	else
		if #skintable == 1 then
			if (not p.spectator or p.mo != nil) and skintable[1] == p.mo.skin then // Do we have this character selected already?
				CONS_Printf(p, "\x82"..SkInfo.name..":".."\x80".." You already have this skin equipped.")
			elseif CV_FindVar("forceskin").value >= 0 then // Do we have a skin restriction active?
				CONS_Printf(p, "\x82"..SkInfo.name..":".."\x80".." Unable to change skins due to skin restriction.")
			else
				CONS_Printf(p, "\x82"..SkInfo.name..":".."\x80".." Found one match, changing to:")
				CONS_Printf(p, SkInfo.skinfostringgen(p, skintable[1], false))
				COM_BufInsertText(p, "skin "..skintable[1][1])
			end
		elseif #skintable > 1 then
			if skID != nil then
				if (not p.spectator or p.mo != nil) and skintable[skID][1] == p.mo.skin then // Do we have this character selected already?
					CONS_Printf(p, "\x82"..SkInfo.name..":".."\x80".." You already have this skin equipped.")
				elseif CV_FindVar("forceskin").value >= 0 then // Do we have a skin restriction active?
					CONS_Printf(p, "\x82"..SkInfo.name..":".."\x80".." Unable to change skins due to skin restriction.")
				else
					CONS_Printf(p, "\x82"..SkInfo.name..":".."\x80".." Changing to:")
					CONS_Printf(p, SkInfo.skinfostringgen(p, skintable[skID], false))
					COM_BufInsertText(p, "skin "..skintable[skID][1])
				end
			else
				CONS_Printf(p, "\x82"..SkInfo.name..":".."\x80".." Multiple skins found with the given search term. Select the desired character by adding the last/red number to command.")
				for k, i in ipairs(skintable) do
					local text = SkInfo.skinfostringgen(p, i, false)
					CONS_Printf(p, text.." \x85"..k)
				end
			end
		else
			CONS_Printf(p, "\x82"..SkInfo.name..":".."\x80".." An non-fatal script error has occured. Aborting function.", skintable, #skintable)
		end
	end
end

// Currently restat?
local function SIIsCurRestat(p)
	if p.HMRs and p.HMRw then
		if ((p.kartspeed == p.HMRs) and (p.kartweight == p.HMRw))
		or ((skins[p.mo.skin].kartspeed == p.HMRs) and (skins[p.mo.skin].kartweight == p.HMRw)) then return true end
	end
end

// Inspecting everyone's current skins.
local function SIInspectSkins(p)
	local restatcvar = CV_FindVar("hm_restat")
	for pi in players.iterate() do
		if not (pi.mo and pi.mo.valid) then continue end
		local intname, realname, speed, weight
		intname, realname = "\x82"..skins[pi.mo.skin].name, "\x82"..skins[pi.mo.skin].realname
		speed, weight = "\x88".."Speed: "..skins[pi.mo.skin].kartspeed, "\x87".."Weight: "..skins[pi.mo.skin].kartweight // Grab current skins' stat.
		// Restat ahoy.
		if restatcvar != nil and restatcvar.value == 1 and SIIsCurRestat(pi) then
			speed, weight = "\x88".."Speed: ".."\x81"..pi.kartspeed, "\x87".."Weight: ".."\x81"..pi.kartweight // We're Restat-ed!
		elseif SkInspectType.value == 1 then
			speed, weight = "\x88".."Speed: "..pi.kartspeed, "\x87".."Weight: "..pi.kartweight // Swap to current player stat instead.
		end
		local text = pi.name..": "..realname.." / "..intname.." "..speed.." "..weight
		local inspt = SkInfo.specialinspect
		local extratext
		for v = 1, #inspt do
			local result = inspt[v](pi)
			if result and result != "" and result != " " then
				if extratext == nil then extratext = result
				else extratext = $.." "..result end
			end
		end
		local extraskinattrib = SkInfo.specialskinattrib
		if #extraskinattrib > 0 then
			for v = 1, #extraskinattrib do
				local result = SkInfo.specialskinattrib[v](skins[pi.mo.skin].name)
				if result != nil then
					if extratext == nil then extratext = result
					else extratext = $.." "..result end
				end
			end
		end
		if extratext != nil and extratext != "" then text = $.."\x84".." "..extratext end
		CONS_Printf(p, text)
	end
end

// Do we have extensions to our script loaded?
local function SIListPlugins(p)
	local plugins = SkInfo.specialskinattrib
	local insplugin = SkInfo.specialinspect
	if (plugins and #plugins > 0) or (insplugin and #insplugin > 0) then
		CONS_Printf(p, "\x82"..SkInfo.name..":".."\x80".." Currently loaded extensions:")
		if plugins and #plugins > 0 then
			for i = 1, #plugins do
				CONS_Printf(p, plugins[i](nil, true))
			end
		end
		if insplugin and #insplugin > 0 then
			for i = 1, #insplugin do
				CONS_Printf(p, insplugin[i](nil, true))
			end
		end
	else
		CONS_Printf(p, "\x82"..SkInfo.name..":".."\x80".." No extensions are loaded.")
	end
end

// Identify and print information about our skin...
// Adding modularity required me to rework the string building function.
// ...that really broke me...
local function SIIdentSkin(p, skin)
	local extraskintext = ""
	local extraskinattrib = SkInfo.specialskinattrib
	if #extraskinattrib > 0 then
		for v = 1, #extraskinattrib do
			local addtext = SkInfo.specialskinattrib[v](skin.name)
			if addtext != nil then
				extraskintext = $ + addtext
			end
		end
	end
	local skininfo = {skin.name, skin.realname, skin.kartspeed, skin.kartweight, #skin, extraskintext}
	CONS_Printf(p, SkInfo.skinfostringgen(p, skininfo, false))
end

// Check if our functions are untouched.
// If they are touched and an admin requests a restore, do it!
local function PerformSFC(restore)
	restore = $ or false
	if restore then
		SkInfo.skinlist = SISkinAccess
		SkInfo.skinstatfilter = SISkinStatFilter
		SkInfo.skinfostringgen = WriteSkinInfoV2
	else
		if SkInfo.skinlist != SISkinAccess
		or SkInfo.skinstatfilter != SISkinStatFilter
		or SkInfo.skinfostringgen != WriteSkinInfoV2 then
			return true
		else return false end
	end
end

// Verification of our exposed functions.
// SkInfo Function Checker, if you will.
local function AccessSFC(p, restore)
	if PerformSFC(false) then
		if restore == "restore" then
			if (IsPlayerAdmin(p) or p == server) then
				// Pull over, that function too fat-- err, not matching.
				PerformSFC(true)
				CONS_Printf(p, "\x82"..SkInfo.name..":".."\x83".." The function(s) has/have been restored.".."\x80")
			else
				CONS_Printf(p, "Only the server or a remote admin can use this.")
			end
		else
			CONS_Printf(p, "\x82"..SkInfo.name..":".."\x85".." The function(s) has/have been modified!".."\x80",
						   "If this is not intended, please contact the adminstrator of this server!")
		end
	else CONS_Printf(p, "\x82"..SkInfo.name..":".."\x80".." The function(s) has/have not been modified.") end
end

// Our main function for the command.
local function MainSI(p, arg1, arg2, arg3, arg4, arg5)
	arg1, arg2, arg3, arg4, arg5 = $ or "", $ or "", $ or "", $ or "", $ or "" // Do default values.
	arg1, arg2, arg3, arg4, arg5 = arg1:lower(), arg2:lower(), arg3:lower(), arg4:lower(), arg5:lower() // oops, all lowercase.
	if arg1 == "list" or arg1 == "l" then
		// We're listing skins... but where?
		// arg3 - desired skin name (full or partial). Performs a search if defined.
		// arg4 - desired stat type.
		// arg5 - desired stat value.
		// Note: If you want to list all characters with a particular stat value, do:
		// skinfo list <console/file> "" <speed/weight/both> <value>
		if arg2 == "console" or arg2 == "c" then
			// Outputting to console, here we go!
			ConsoleSkinList(p, arg3, arg4, arg5)
		elseif arg2 == "file" or arg2 == "f" then
			// Guess we're writing to file...
			local filename = "skinlist.txt"
			SIOutputFile(p, filename, arg3, arg4, arg5)
		else
			CONS_Printf(p, "\x82"..SkInfo.name..":".."\x80".." Invalid syntax.")
		end
	elseif arg1 == "forceskin" or arg1 == "fskin" or arg1 == "fs" then
		// This section almost didn't happen. ForceSkin help ahoy!
		// arg2 - desired skin name. Performs search if defined.
		// arg3 - entry. Used if more than one result appears from search.
		if arg2 != "" then
			if (IsPlayerAdmin(p) or p == server) then SIForceSkin(p, arg2, arg3)
			else CONS_Printf(p, "Only the server or a remote admin can use this.") end
		else
			local skinid = CV_FindVar("forceskin").value
			if skinid >= 0 then
				CONS_Printf(p, "\x82"..SkInfo.name..":".."\x80".." The current skin restriction is set to:")
				SIIdentSkin(p, skins[skinid])
			else CONS_Printf(p, "\x82"..SkInfo.name..":".."\x80".." Skin restriction is not currently in place.") end
		end
	elseif arg1 == "skin" or arg1 == "s" then
		// Make it a bit easier to change skins ourselves.
		// arg2 - desired skin name. Performs search if defined.
		// arg3 - entry. Used if more than one result appears from search.
		if arg2 != "" then
			SISkin(p, arg2, arg3)
		else
			// Catch our dedicated buddies.
			if p == server and isdedicatedserver then error("You shouldn't be doing this from console of a dedicated server.") end
			// List info about our current skin.
			if p.spectator or p.mo == nil then
				CONS_Printf(p, "\x82"..SkInfo.name..":".."\x80".." Cannot identify skin used. Are you spectating?")
			else
				CONS_Printf(p, "\x82"..SkInfo.name..":".."\x80".." Your current skin is:")
				SIIdentSkin(p, skins[p.mo.skin])
			end
		end
	elseif arg1 == "inspect" then
		// View everyone's skins!
		SIInspectSkins(p, arg2)
	elseif arg1 == "verify" or arg1 == "v" then
		// Did someone change the function(s) to be something else?!
		AccessSFC(p, arg2)
	elseif arg1 == "plugin" or arg1 == "plugins" or arg1 == "p" or arg1 == "extensions" then
		// We got more stuff attached onto us!
		SIListPlugins(p)
	else
		// List our commands!
		if arg1 == nil or arg1 == "" then
			CONS_Printf(p, "\x82"..SkInfo.fullname.." "..SkInfo.ver.."\x80".." by ".."\x87"..SkInfo.author.."\x80",
						   "A character/skin-related library/script with listing/searching functionality.")
			local cmdlist = {
				{"skinfo list <console/file> [skinname]", " - search through/list all currently present characters to console or file."},
				{"skinfo forceskin [skinname/off/-1] [entry]", " - an extension to vanilla 'forceskin'. Can search for characters. If no arguments are given, prints information on skin being restricted to."},
				{"skinfo skin [skinname] [entry]", " - change your current skin. Can search for characters. If no arguments are given, prints information on current skin used."},
				{"skinfo verify [restore]", " - check if the script's functions are altered. If so, administrators can [restore] it back to normal."},
				{"skinfo plugins", " - check if any extensions are loaded."},
				{"skinfo inspect", " - view skins, speed and weight of all players."},
				{"skinfoinspecttype <skin/player>", " - determines whether speed/weight stats are read from skin or player."},
				{"skpluginrun <0-1>", " - CVar whether to use plugins loaded or not."}
			}
			for i = 1, #cmdlist do
				CONS_Printf(p, "\x82"..cmdlist[i][1].."\x80"..cmdlist[i][2])
			end
		else
			CONS_Printf(p, "\x82"..SkInfo.name..":".."\x80".." Invalid syntax.")
		end
	end
end

// Different ways of accessing.
COM_AddCommand("skinfo", MainSI)
COM_AddCommand("sk", MainSI)

// We are loaded! Print our presence!
print("\x82"..SkInfo.fullname.." "..SkInfo.ver.."\x80".." by ".."\x87"..SkInfo.author.."\x80")

// Plugin integration from PluginExample.
// Here, we check if the skin is using "Simple AnimaL".
// Listing it is SISimpleAnimaL(nil, true).
local function SISimpleAnimaL(intname, name, p)
	if name then return "Simple AnimaL Detection" end
	if _G["SIMPLE_ANIMAL_VERSION"] and _G["SIMPLE_ANIMAL_DEFINITIONS"]
	and _G["SIMPLE_ANIMAL_DEFINITIONS"][intname] then
		return "(Simple AnimaL)" // Make our presence known!
	end
end
table.insert(SkInfo.specialskinattrib, SISimpleAnimaL)

// Inspect-specific stuff.
local function SIInspectDitto(ip, name)
	if name then return "Ditto Inspect" end
	if ip.is_actually_a_ditto then return "(Ditto)" end
end
local function SIInspectRestat(ip, name)
	if name then return "Restat Inspect" end
	local restatcvar = CV_FindVar("hm_restat")
	if restatcvar != nil and restatcvar.value == 1 and ip.HMRs and ip.HMRw then return "(Restat S"..ip.HMRs.." W"..ip.HMRw..")" end
end
table.insert(SkInfo.specialinspect, SIInspectDitto)
table.insert(SkInfo.specialinspect, SIInspectRestat)